import geompy
import math
import inspect
import os
import SALOMEDS

### for mesher
import salome, SMESH
import smesh
import NETGENPlugin
  

Bbot1 = geompy.MakeBoxDXDYDZ(3, 5, 2)
[Face_1,Face_2,Face_3,Face_4,Face_5,Face_6] = geompy.SubShapeAllSorted(Bbot1, geompy.ShapeType["FACE"])

# vertices for cutting box
# walls of 0.2 mm remain after cutting
r1 = geompy.MakeVertex(0.2, 0.2, 0)
r2 = geompy.MakeVertex(2.8, 0.2, 0)
r3 = geompy.MakeVertex(2.8, 4.8, 0)
r4 = geompy.MakeVertex(0.2, 4.8, 0)

Lr1 = geompy.MakeLineTwoPnt(r1, r2)
Lr2 = geompy.MakeLineTwoPnt(r2, r3)
Lr3 = geompy.MakeLineTwoPnt(r3, r4)
Lr4 = geompy.MakeLineTwoPnt(r4, r1)

z2 = geompy.MakeVertex(0, 0, 2)  # define vector Zstart
z0 = geompy.MakeVertex(0, 0, 0)  # define vector Zend
vz = geompy.MakeVector(z0, z2)   # define vector 

# define internal block
Rface = geompy.MakeFaceWires([Lr1, Lr2, Lr3, Lr4], 1)
Bint2 = geompy.MakePrismVecH(Rface, vz, 1.8)

# cut away internal block from solid block Bbot1
# a shoe box Bshoe remains
Bshoe = geompy.MakeCut(Bbot1, Bint2)

# define points for the two cylinders on top of Bshoe
# centre point for circle
# define points top plate
# vertices for top plane
p1 = geompy.MakeVertex(1.5, 1.5, 2)
p2 = geompy.MakeVertex(1.5, 3.5, 2)
q1 = geompy.MakeVertex(0.2, 0.2, 2)
q2 = geompy.MakeVertex(2.8, 0.2, 2)
q3 = geompy.MakeVertex(2.8, 4.8, 2)
q4 = geompy.MakeVertex(0.2, 4.8, 2)
Lq1 = geompy.MakeLineTwoPnt(q1, q2)
Lq2 = geompy.MakeLineTwoPnt(q2, q3)
Lq3 = geompy.MakeLineTwoPnt(q3, q4)
Lq4 = geompy.MakeLineTwoPnt(q4, q1)

Rc1  = 0.7
Rc2  = 0.5
Hcyl = 0.8
Htop = 0.6
Circle1 = geompy.MakeCircle(p1,vz,Rc1)
Circle2 = geompy.MakeCircle(p2,vz,Rc2)
Fcircle = geompy.MakeFaceWires([Circle1,Circle2],1)
Cyl12 = geompy.MakePrismVecH(Fcircle,vz,Hcyl)
rectq = geompy.MakeFaceWires([Circle1, Circle2, Lq1, Lq2, Lq3, Lq4], 1)
Btop = geompy.MakePrismVecH(rectq,vz,Htop)

# now all solids are defined:
# Bshoe, Btop and Cyl12 then 
# fuse Fshoecyl <-- Bshoe and Cyl12 and 
# fuse Fassembl <-- Fshoecyl and Btop
Fshoecyl = geompy.MakeFuse(Bshoe, Cyl12)
Fassembl = geompy.MakeFuse(Fshoecyl, Btop)


# now define planes for partitioning the construction
# define axial area of cylinders
Hcyl  =1.5 # make sure height of cylinder area is more than height of cylinder
PCyl1 = geompy.MakePrismVecH(Circle1,vz,Hcyl)
PCyl2 = geompy.MakePrismVecH(Circle2,vz,Hcyl)

# plane P12 between Bshoe and Btop
Psize = 5.0    # make sure Psize is large enough to cover whole construction
q12 = geompy.MakeVertex(1.5, 2.5, 2.0)   #0.25*(p1+q2+q3+q4)
P12 = geompy.MakePlane(q12, vz, Psize)  


# create partition of the four fused shapes
# by dividing them on their surfaces
# Pcentre: plane between Bshoe and upper three shape
# Cyl1,2: cylinder area between cylinders1,2 and Btop
listSubShapeIDs = geompy.SubShapeAllIDs(Fassembl, geompy.ShapeType["SOLID"])
Partit=geompy.MakePartition([Fassembl],[P12,PCyl1,PCyl2],[],[],geompy.ShapeType["SOLID"],0,[],0)

BPshoe = geompy.CreateGroup(Partit, geompy.ShapeType["SOLID"])
geompy.UnionIDs(BPshoe, [2])
Partit = geompy.GetMainShape(BPshoe)
BPtop = geompy.CreateGroup(Partit, geompy.ShapeType["SOLID"])
geompy.UnionIDs(BPtop, [88])
Partit = geompy.GetMainShape(BPtop)
BPcyl1 = geompy.CreateGroup(Partit, geompy.ShapeType["SOLID"])
geompy.UnionIDs(BPcyl1, [124])
Partit = geompy.GetMainShape(BPcyl1)
BPcyl2 = geompy.CreateGroup(Partit, geompy.ShapeType["SOLID"])
geompy.UnionIDs(BPcyl2, [133])
Partit = geompy.GetMainShape(BPcyl2)


geompy.addToStudy( p1, "p1" )
geompy.addToStudy( p2, "p2" )
geompy.addToStudy( z0, "z0" )
geompy.addToStudy( z2, "z2" )
geompy.addToStudy( vz, "vz" )

geompy.addToStudy( Circle1, "Circle1" )
geompy.addToStudy( Circle2, "Circle2" )
geompy.addToStudy( Fcircle, "Fcircle" )

geompy.addToStudy( r1, "r1" )
geompy.addToStudy( r2, "r2" )
geompy.addToStudy( r3, "r3" )
geompy.addToStudy( r4, "r4" )
geompy.addToStudy( Lr1, "Lr1" )
geompy.addToStudy( Lr2, "Lr2" )
geompy.addToStudy( Lr3, "Lr3" )
geompy.addToStudy( Lr4, "Lr4" )

geompy.addToStudy( q1, "q1" )
geompy.addToStudy( q2, "q2" )
geompy.addToStudy( q3, "q3" )
geompy.addToStudy( q4, "q4" )


geompy.addToStudy(P12, "P12" )
geompy.addToStudy(q12, "q12" )

geompy.addToStudy( Lq1, "Lq1" )
geompy.addToStudy( Lq2, "Lq2" )
geompy.addToStudy( Lq3, "Lq3" )
geompy.addToStudy( Lq4, "Lq4" )

geompy.addToStudy( rectq, "rectq" )
geompy.addToStudy( Btop,  "Btop" )
geompy.addToStudy( Bbot1, "Bbot1" )
geompy.addToStudy( Bint2, "Bint2" )
geompy.addToStudy( Bshoe,  "Bshoe" )

geompy.addToStudy( Cyl12, "Cyl12" )
geompy.addToStudy( PCyl1, "PCyl1" )
geompy.addToStudy( PCyl2, "PCyl2" )
geompy.addToStudy( Fshoecyl, "Fshoecyl")
geompy.addToStudy( Fassembl,"Fassembl")
geompy.addToStudy( Partit, "Partit")

geompy.addToStudyInFather( Partit, BPshoe, "BPshoe" )
geompy.addToStudyInFather( Partit, BPtop, "BPtop" )
geompy.addToStudyInFather( Partit, BPcyl1, "BPcyl1" )
geompy.addToStudyInFather( Partit, BPcyl2, "BPcyl2" )

NETGEN_3D_Parameters = smesh.smesh.CreateHypothesis('NETGEN_Parameters', 'NETGENEngine')
NETGEN_3D_Parameters.SetMaxSize( 0.10 )   ### 0.65
NETGEN_3D_Parameters.SetSecondOrder( 1 )
NETGEN_3D_Parameters.SetOptimize( 1 )
NETGEN_3D_Parameters.SetFineness( 2 )     ### 1 course - 2 moderate
MPartit = smesh.Mesh(Partit)
status = MPartit.AddHypothesis(NETGEN_3D_Parameters)
Netgen_1D_2D_3D = smesh.smesh.CreateHypothesis('NETGEN_2D3D', 'NETGENEngine')
status = MPartit.AddHypothesis(Netgen_1D_2D_3D)
isDone = MPartit.Compute()

BPshoe = MPartit.Group(BPshoe)
BPtop  = MPartit.Group(BPtop)
BPcyl1 = MPartit.Group(BPcyl1)
BPcyl2 = MPartit.Group(BPcyl2)

#BPbot.SetColor(SALOMEDS.Color(0.384314,1,0.796078))
#BPtop.SetColor(SALOMEDS.Color(0.384314,1,0.796078))
#BPcyl1.SetColor(SALOMEDS.Color(0.384314,1,0.796078))
#BPcyl2.SetColor(SALOMEDS.Color(0.384314,1,0.796078))

## set object names  
smesh.SetName(NETGEN_3D_Parameters, 'NETGEN_3D_Parameters')
smesh.SetName(MPartit.GetMesh(),'MPartit')
#smesh.SetName(NETGEN_2D3D_1, 'NETGEN_2D3D_1')
smesh.SetName(BPshoe, 'BPshoe')
smesh.SetName(BPtop,  'BPtop')
smesh.SetName(BPcyl1, 'BPcyl1')
smesh.SetName(BPcyl2, 'BPcyl2')

## save mesh MPartit at current directory
cfilename =  inspect.getfile(inspect.currentframe())
cdir = os.path.dirname(cfilename)
MeshFile = 'MPartit.med'
PathFileName = os.path.join(cdir,MeshFile)
MPartit.ExportMED(PathFileName, 0, SMESH.MED_V2_2, 1 )

if salome.sg.hasDesktop():
  salome.sg.updateObjBrowser(1)



